// combinationMachine.js
//
// Takes two parameters prepended with 'combinationMachine'.
// Produces all the possible binary combinations given a
// certain length and amount of maximum simultaneous 
// connections. It prepends this with the index of each
// combination.
// 
// Outlet 1 = a stream of arrays
// Outlet 2 = total amount of arrays
//

// define inlets and outlets
inlets = 1;
outlets = 2;

// master function which calculates all permutations and spits them out to Max6
function combinationMachine(n, choose) {
	var combinations = getCombinationsUpTo(arrayOfIndices(n), choose);
	var formatted = formatCombinations(combinations, n);
	
	for (var i = 0; i < formatted.length; i++) {
		outlet(0, i, formatted[i]);
	}
	
	outlet(1, formatted.length);
}

// create an array of incides of arbitrary length
function arrayOfIndices(length) {
    var results = [];

    for (var i = 0; i < length; i++) {
        results.push(i);
    }
    return results
}

// calculate all combination UP TO the total number of simultaneous connections
function getCombinationsUpTo(options, n) {
	var combinations = new Array();
	
	for (var i = n; i > 0; i--) {
		// use a deep copy of options since getCombination obliterates it!
		var _options = options.slice();
		
		combinations = combinations.concat(getCombinations(_options, i));
	}
	
	return combinations;
}

// produce all the combinations of n simultaneous connections
// in an array of a given length
function getCombinations(options, n) {
    var combinations = [];

    if (n > 1) {
        // middle of the recursion...

        // 1. for each position, in sequence  
        while (options.length >= n) {
            var currentPosition = options.shift();
            var children = getCombinations(options.slice(), n - 1);

            // 2. get all the possible "child" combinations
            while (children.length) {
                var currentCombination = children.shift();

                // 3. and shove them onto an array 
                var container = [];
                container.push(currentPosition);

                if (currentCombination.length) {
                    // 3a. by flattening the children, if an array, or
                    for (var i = 0; i < currentCombination.length; i++) {
                        container.push(currentCombination[i]);
                    }
                } else {
                    // 3b. by adding them directly, if values
                    container.push(currentCombination);
                }

                combinations.push(container);
            }
        }

    } else {
        // end of the recursion
        // just box up each option...

        for (var i = 0; i < options.length; i++) {
            combinations.push([options[i]]);
        }
    }

    return combinations;
}

// convert indices to a bitmap
function indicesToBitmap(indices) {
    var value = 0;
    
    for (var i = 0; i < indices.length; i++) {
        value |= (1 << indices[i]);
    }
    
    return value;
}

// break bitmap into an array with spaces between each value
function addSpaces(array) {
	return array.split('').join(' ');
}

// format the array by padding with zeros and adding spaces to the output
function formatCombinations(combinations, maxLength) {    
    var padding = new Array(maxLength).join('0');
	var formatted = [];
        
    for (var i = 0; i < combinations.length; i++) {
        var asBitString = indicesToBitmap(combinations[i]).toString(2);
        var combined = padding + asBitString;       
        var padded = combined.substr(combined.length - maxLength);
        
        formatted.push(addSpaces(padded));
    }
	
	return formatted;
}

/* for testing outside of max...
	function outlet(number, args) {
		console.log("outlet " + number + ": " + args);
	}
*/
